#ifndef _COURSECOLLECTION_CPP
#define _COURSECOLLECTION_CPP

#include <string>
#include <fstream>
#include <algorithm>

#include "MyTmpltLib.h"

#include "CourseCollection.h"

using namespace std;

CourseCollection::CourseCollection()
{
   try
   {
      ccReserved = ccIncr;
      ccArr = new Course[ ccReserved ];
      ccNOfCourses = 0;
   }
   catch( bad_alloc& )
   {
      throw CourseCollectionXptn( "CourseCollection",
                                  CourseCollectionXptn::allocFailed);
   }
} // CourseCollection::CourseCollection

void CourseCollection::erase1Course( int ndx )
{
   ccArr[ndx] = ccArr[ccNOfCourses-1];
   --ccNOfCourses;
} // CourseCollection::erase1Course

const Course& CourseCollection::get1Course( string code ) const
{
   int ndx( findNdx(code) );
   if ( ndx < 0 )
      throw CourseCollectionXptn( "get1Course",
                                  CourseCollectionXptn::notFound,
                                  code.c_str() );
   return ccArr[ndx];
} // CourseCollection::get1Course

void CourseCollection::save( ofstream& bout ) const
{
   if ( bout.fail() )
      throw  CourseCollectionXptn( "save",
                                   CourseCollectionXptn::fileNotOpen );
   bout.write( reinterpret_cast<const char*>(&ccNOfCourses),
               sizeof(ccNOfCourses) );
   for ( int k(0); k < ccNOfCourses; ++k )
      ccArr[k].save( bout );
   if ( bout.fail() )
      throw  CourseCollectionXptn( "save",
                                   CourseCollectionXptn::cannotWrite );
} // CourseCollection::save

void CourseCollection::load( ifstream& bin )
{
   CourseCollection tmp;
   unsigned int n;

   bin.read( reinterpret_cast<char*>(&n), sizeof(ccNOfCourses) );
   if ( !bin.eof() )
   {
      for ( int k(0); k < n && !bin.fail(); ++k )
      {
         Course oneCourse;
         oneCourse.load( bin );
         tmp.insert1Course( oneCourse );
      }
      if ( bin.fail() )
         throw  CourseCollectionXptn( "load",
                                   CourseCollectionXptn::cannotRead );
      swap( tmp );
   }
} // CourseCollection::load

void CourseCollection::swap( CourseCollection& rhs )
{
   Course* sv( ccArr );
   ccArr = rhs.ccArr;  rhs.ccArr = sv;
   std::swap( ccNOfCourses, rhs.ccNOfCourses );
   std::swap( ccReserved, rhs.ccReserved ); 
} // CourseCollection::swap

void CourseCollection::display( ostream& tout ) const
{
   for ( int k(0); k < ccNOfCourses; ++k )
      ccArr[k].display( tout );
}

int CourseCollection::findNdx( const string& code ) const
{
   int ndx;
   if ( code.length() != Course::cCodeSz-1 )
      ndx = -1;
   else
      ndx = linSearch( ccArr, ccNOfCourses,
                       0, ccNOfCourses-1, Course(code) );
   return ndx;
} // CourseCollection::findNdx

void CourseCollection::insert1Course( const Course& aCourse )
{
   if ( ccReserved <= ccNOfCourses+1 )
   {
      try {  renew( ccArr, ccNOfCourses, ccReserved+ccIncr );
             ccReserved += ccIncr;  }
      catch( MyTmpltLibXptn& )
      {
         throw CourseCollectionXptn( "insert1Course",
                                     CourseCollectionXptn::allocFailed );
      }
   }
   ccArr[ccNOfCourses] = aCourse;
   ++ccNOfCourses; 
} // CourseCollection::insert1Course

void CourseCollection::add1Course( const Course& aCourse )
{
   if ( strlen(aCourse.getCode()) != Course::cCodeSz-1 )
      throw CourseCollectionXptn( "add1Course",
                                  CourseCollectionXptn::entity );
   int ndx( findNdx(aCourse.getCode()) ); 
   if ( ndx < 0 ) //    
   {
      if ( strcmp(aCourse.getPrereq(), "") != 0 )   // 
      {                                             // 
         int ndx( findNdx(aCourse.getPrereq()) );
         if ( ndx < 0 ) //      
            throw CourseCollectionXptn( "add1Course",
                                  CourseCollectionXptn::prereqRef,
                                        aCourse.getPrereq() );
      }
   //         
      insert1Course( aCourse );
      ndx = findNdx( aCourse.getCode() );
      ccArr[ndx].clearStudents();
   }
} // CourseCollection::add1Course

void CourseCollection::delete1Course( string code )
{
   int ndx( findNdx(code) );
   if ( ndx >= 0 )  // 
   {
      ccArr[ccNOfCourses] = Course();
      ccArr[ccNOfCourses].setPrereq( code ); // 
      int k(0);
      while ( strcmp(ccArr[k].getPrereq(), code.c_str()) != 0 )
         ++k;
      if ( k < ccNOfCourses )
         throw CourseCollectionXptn( "delete1Course",
                                  CourseCollectionXptn::prereqRef,
                                     code.c_str() );
      int enrStdnt( ccArr[ndx].getNoOfStudents() );
      if ( enrStdnt > 0 ) //   
         throw CourseCollectionXptn( "delete1Course",
                                  CourseCollectionXptn::enrollRef,
                                         code.c_str(), enrStdnt );
      erase1Course( ndx );
   }
} // CourseCollection::delete1Course

void CourseCollection::add1Student( string code )
{
   int ndx( findNdx(code) );
   if ( ndx < 0 )
      throw CourseCollectionXptn( "add1Student",
                                  CourseCollectionXptn::notFound,
                                  code.c_str() );
   ccArr[ndx].add1Student();
} // CourseCollection::add1Student

void CourseCollection::delete1Student( string code )
{
   int ndx( findNdx(code) );
   if ( ndx < 0 )
      throw CourseCollectionXptn( "delete1Student",
                                  CourseCollectionXptn::notFound,
                                  code.c_str() );
   ccArr[ndx].delete1Student();
} // CourseCollection::delete1Student

#endif // _COURSECOLLECTION_CPP

